@@ -149,6 +149,11 @@ ILLEGAL_VOL = "ILLEGAL"
 LEGAL_VOL = "LEGAL"
 FAKE_VOL = "FAKE"
 
+# Volume info statuses
+VOL_STATUS_OK = "OK"
+VOL_STATUS_INIT = "INIT"
+VOL_STATUS_INVALID = "INVALID"
+
 FMT2STR = {
     COW_FORMAT: qemuimg.FORMAT.QCOW2,
     RAW_FORMAT: qemuimg.FORMAT.RAW,
