@@ -68,13 +68,13 @@ if backup_enabled:
         "backupBegin",
         "abortJob",
         "backupGetXMLDesc",
-        "checkpointCreateXML",
+        "checkpointLookupByName",
         "blockInfo"
     )
     class DomainAdapter(object):
         """
-        VM wrapper class that expose only
-        libvirt backup related operations
+        VM wrapper class that exposes only
+        libvirt backup related operations.
         """
         def __init__(self, vm):
             self._vm = vm
@@ -114,12 +114,6 @@ class BackupConfig(properties.Owner):
 def start_backup(vm, dom, config):
     backup_cfg = BackupConfig(config)
 
-    if backup_cfg.from_checkpoint_id is not None:
-        raise exception.BackupError(
-            reason="Incremental backup not supported yet",
-            vm_id=vm.id,
-            backup=backup_cfg)
-
     try:
         drives = _get_disks_drives(vm, backup_cfg.disks)
     except LookupError as e:
@@ -137,7 +131,8 @@ def start_backup(vm, dom, config):
 
     try:
         vm.freeze()
-        backup_xml = create_backup_xml(nbd_addr, drives, scratch_disks)
+        backup_xml = create_backup_xml(
+            nbd_addr, drives, scratch_disks, backup_cfg.from_checkpoint_id)
         checkpoint_xml = create_checkpoint_xml(backup_cfg, drives)
 
         vm.log.info(
@@ -160,7 +155,13 @@ def start_backup(vm, dom, config):
         img_id: nbd_addr.url(drive.name)
         for img_id, drive in six.iteritems(drives)}
 
-    return {'result': {'disks': disks_urls}}
+    result = {'disks': disks_urls}
+
+    if backup_cfg.to_checkpoint_id is not None:
+        _add_checkpoint_xml(
+            vm, dom, backup_cfg.backup_id, backup_cfg.to_checkpoint_id, result)
+
+    return dict(result=result)
 
 
 def stop_backup(vm, dom, backup_id):
@@ -185,12 +186,17 @@ def stop_backup(vm, dom, backup_id):
     _remove_scratch_disks(vm, backup_id)
 
 
-def backup_info(vm, dom, backup_id):
+def backup_info(vm, dom, backup_id, checkpoint_id=None):
     backup_xml = _get_backup_xml(vm.id, dom, backup_id)
     vm.log.debug("backup_id %r info: %s", backup_id, backup_xml)
 
     disks_urls = _parse_backup_info(vm, backup_id, backup_xml)
-    return {'result': {'disks': disks_urls}}
+    result = {'disks': disks_urls}
+
+    if checkpoint_id is not None:
+        _add_checkpoint_xml(vm, dom, backup_id, checkpoint_id, result)
+
+    return dict(result=result)
 
 
 def delete_checkpoints(vm, dom, checkpoint_ids):
@@ -230,6 +236,21 @@ def _get_backup_xml(vm_id, dom, backup_id):
     return backup_xml
 
 
+def _add_checkpoint_xml(vm, dom, backup_id, checkpoint_id, result):
+    try:
+        checkpoint = dom.checkpointLookupByName(checkpoint_id)
+        result['checkpoint'] = checkpoint.getXMLDesc()
+    except libvirt.libvirtError as e:
+        if e.get_error_code() == libvirt.VIR_ERR_NO_DOMAIN_CHECKPOINT:
+            vm.log.exception(
+                "Checkpoint_id: %r for backup_id: %r, doesn't exist, "
+                "error: %s", checkpoint_id, backup_id, e)
+        else:
+            vm.log.exception(
+                "Failed to fetch checkpoint_id: %r for backup_id: %r, "
+                "error: %s", checkpoint_id, backup_id, e)
+
+
 def _begin_backup(vm, dom, backup_cfg, backup_xml, checkpoint_xml):
     # pylint: disable=no-member
     flags = libvirt.VIR_DOMAIN_BACKUP_BEGIN_REUSE_EXTERNAL
@@ -292,9 +313,14 @@ def _raise_parse_error(vm_id, backup_id, backup_xml):
         backup_id=backup_id)
 
 
-def create_backup_xml(address, drives, scratch_disks):
+def create_backup_xml(address, drives, scratch_disks, from_checkpoint_id=None):
     domainbackup = vmxml.Element('domainbackup', mode='pull')
 
+    if from_checkpoint_id is not None:
+        incremental = vmxml.Element('incremental')
+        incremental.appendTextNode(from_checkpoint_id)
+        domainbackup.appendChild(incremental)
+
     server = vmxml.Element(
         'server', transport=address.transport, socket=address.path)
 
